@@ -1,10 +1,10 @@
-# $Id: Application.pm,v 1.15 2001/06/25 03:01:19 jesse Exp $
+# $Id: Application.pm,v 1.22 2002/05/06 03:10:43 jesse Exp $
 
 package CGI::Application;
 
 use strict;
 
-$CGI::Application::VERSION = '2.0';
+$CGI::Application::VERSION = '2.2';
 
 
 use CGI;
@@ -98,7 +98,12 @@ sub run {
 
 	# If $rm undefined, use default (start) mode
 	my $def_rm = $self->start_mode() || '';
-	$rm = $def_rm unless (defined($rm));
+	$rm = $def_rm unless (defined($rm) && length($rm));
+
+	# Call PRE-RUN hook, now that we know the run-mode
+	# This hook can be used to provide run-mode specific behaviors
+	# before the run-mode actually runs.
+	$self->cgiapp_prerun($rm);
 
 	my %rmodes = ($self->run_modes());
 
@@ -164,11 +169,20 @@ sub run {
 
 sub cgiapp_init {
 	my $self = shift;
+	my @args = (@_);
 
 	# Nothing to init, yet!
 }
 
 
+sub cgiapp_prerun {
+	my $self = shift;
+	my $rm = shift;
+
+	# Nothing to prerun, yet!
+}
+
+
 sub setup {
 	my $self = shift;
 
@@ -320,23 +334,36 @@ sub mode_param {
 
 sub param {
 	my $self = shift;
-	my ($param, $value) = @_;
+	my (@data) = (@_);
 
 	# First use?  Create new __PARAMS!
 	$self->{__PARAMS} = {} unless (exists($self->{__PARAMS}));
 
 	my $rp = $self->{__PARAMS};
 
-	# Return the list of param keys if no param is specified.
-	return (keys(%$rp)) unless(defined($param));
-
-	# If a value is specified, set it!
-	if (defined($value)) {
-		$rp->{$param} = $value;
+	# If data is provided, set it!
+	if (scalar(@data)) {
+		# Is it a hash, or hash-ref?
+		if (ref($data[0]) eq 'HASH') {
+			# Make a copy, which augments the existing contents (if any)
+			%$rp = (%$rp, %{$data[0]});
+		} elsif ((scalar(@data) % 2) == 0) {
+			# It appears to be a possible hash (even # of elements)
+			%$rp = (%$rp, @data);
+		} elsif (scalar(@data) > 1) {
+			croak("Odd number of elements passed to param().  Not a valid hash");
+		}
+	} else {
+		# Return the list of param keys if no param is specified.
+		return (keys(%$rp));
 	}
 
-	# If we've gotten this far, return the param value!
-	return $rp->{$param};
+	# If exactly one parameter was sent to param(), return the value
+	if (scalar(@data) <= 2) {
+		my $param = $data[0];
+		return $rp->{$param};
+	}
+	return;  # Otherwise, return undef
 }
 
 
@@ -375,13 +402,13 @@ sub run_modes {
 	if (scalar(@data)) {
 		# Is it a hash, or hash-ref?
 		if (ref($data[0]) eq 'HASH') {
-			# Make a copy
-			%$rr_m = %{$data[0]};
+			# Make a copy, which augments the existing contents (if any)
+			%$rr_m = (%$rr_m, %{$data[0]});
 		} elsif ((scalar(@data) % 2) == 0) {
 			# It appears to be a possible hash (even # of elements)
-			%$rr_m = @data;
+			%$rr_m = (%$rr_m, @data);
 		} else {
-			croak("Odd number of elements passed to run_modes().  Not a valid hash")
+			croak("Odd number of elements passed to run_modes().  Not a valid hash");
 		}
 	}
 
@@ -490,6 +517,10 @@ Framework for building reusable web-applications
 	my $self = shift;
 	# Optional Pre-setup initalization behaviors
   }
+  sub cgiapp_prerun {
+	my $self = shift;
+	# Optional Pre-runmode initalization behaviors
+  }
   sub teardown {
 	my $self = shift;
 	# Optional Post-response shutdown behaviors
@@ -753,12 +784,18 @@ PARAMS        - This parameter, if used, allows you to set a number
 of custom parameters at run-time.  By passing in different 
 values in different instance scripts which use the same application 
 module you can achieve a higher level of reusability.  For instance, 
-imagine an application module, "MailForm.pm".  The application takes 
+imagine an application module, "Mailform.pm".  The application takes 
 the contents of a HTML form and emails it to a specified recipient.
 You could have multiple instance scripts throughout your site which 
-all use this "MailForm.pm" module, but which set different recipients
+all use this "Mailform.pm" module, but which set different recipients
 or different forms.
 
+QUERY         - This optional parameter allows you to specify an 
+already-created CGI.pm query object.  Under normal use, 
+CGI::Application will instantiate its own CGI.pm query object.
+Under certain conditions, it might be useful to be able to use
+one which has already been created.
+
 
 =item run()
 
@@ -841,7 +878,7 @@ all the arguments which were sent to the new() method.
 
 An example of the benefits provided by utilizing this hook is 
 creating a custom "application super-class" from which which all 
-your CGI applicatons would inherit, instead of CGI::Application.
+your CGI applications would inherit, instead of CGI::Application.
 
 Consider the following:
 
@@ -869,6 +906,45 @@ characteristics.  This has the potential for much cleaner code
 built on object-oriented inheritance.
 
 
+=item cgiapp_prerun()
+
+If implemented, this method is called automatically right before the
+selected run-mode method is called.  This method provides an optional
+pre-runmode hook, which permits functionality to be added at the point
+right before the run-mode method is called.  To further leverage this
+hook, the value of the run-mode is passed into cgiapp_prerun().
+  
+Another benefit provided by utilizing this hook is
+creating a custom "application super-class" from which all
+your CGI applications would inherit, instead of CGI::Application.
+
+Consider the following:
+
+  # In MySuperclass.pm:
+  package MySuperclass;
+  use base 'CGI::Application';
+  sub cgiapp_prerun {
+	my $self = shift;
+	# Perform some project-specific init behavior
+	# such as to implement run-mode specific
+	# authorization functions.
+  }
+
+
+  # In MyApplication.pm:
+  package MyApplication;
+  use base 'MySuperclass';
+  sub setup { ... }
+  sub teardown { ... }
+  # The rest of your CGI::Application-based follows...  
+
+
+By using CGI::Application and the cgiapp_prerun() method as illustrated, 
+a suite of applications could be designed to share certain 
+characteristics.  This has the potential for much cleaner code 
+built on object-oriented inheritance.
+
+
 =back
 
 
@@ -986,8 +1062,6 @@ be used as the value of the current run-mode.  E.g., a "mode param method":
 This would allow you to programmatically set the run-mode based on something 
 besides the value of a CGI parameter -- $ENV{PATH_INFO}, for example.
 
-(Thanks to Stephen Howard for this idea!)
-
 
 
 =item param()
@@ -1010,6 +1084,15 @@ currently exist:
 
     my @all_params = $webapp->param();
 
+The param() method also allows you to set a bunch of parameters at once
+by passing in a hash (or hashref):
+
+    $webapp->param(
+        'key1' => 'val1',
+        'key2' => 'val2',
+        'key3' => 'val3',
+    );
+
 The param() method enables a very valuable system for 
 customizing your applications on a per-instance basis.  
 One Application Module might be instantiated by different 
@@ -1041,7 +1124,8 @@ interact with form data.
 
 When the new() method is called, a CGI query object is automatically created.
 If, for some reason, you want to use your own CGI query object, the new()
-method supports passing in your existing query object on construction.
+method supports passing in your existing query object on construction using
+the QUERY attribute.
 
 
 =item run_modes()
@@ -1081,6 +1165,13 @@ is performance.  Dereferencing a subref is faster than eval()-ing
 a code block.  If run-time performance is a significant issue, specify
 your run-mode methods by reference and not by name.
 
+The run_modes() method may be called more than once.  Additional values passed 
+into run_modes() will be added to the run-modes table.  In the case that an 
+existing run-mode is re-defined, the new value will override the existing value.
+This behavior might be useful for applications which are created via inheritance 
+from another application, or some advanced application which modifies its
+own capabilities based on user input.
+
 
 B<IMPORTANT NOTE ABOUT RUN-MODE METHODS>
 
@@ -1094,7 +1185,7 @@ sending content before your HTTP header, you are probably breaking this rule.
 B<THE RUN-MODE OF LAST RESORT: "AUTOLOAD">
 
 If CGI::Application is asked to go to a run-mode which doesn't exist
-if will usually croak() with errors.  If this is not your desired 
+it will usually croak() with errors.  If this is not your desired 
 behavior, it is possible to catch this exception by implementing 
 a run-mode with the reserved name "AUTOLOAD":
 
@@ -1162,6 +1253,19 @@ post them to the support mailing list!  To join the mailing list, simply
 send a blank message to "cgiapp-subscribe@lists.vm.com".
 
 
+B<More Reading>
+
+If you're interested in finding out more about CGI::Application, the 
+following article is available on Perl.com:
+
+    Using CGI::Application
+    http://www.perl.com/pub/a/2001/06/05/cgi.html
+
+Thanks to Simon Cozens and the O'Reilly network for publishing this
+article, and for the incredible value they provide to the Perl
+community!
+
+
 =head1 CREDITS
 
 Thanks go to my place of work, Vanguard Media (http://www.vm.com),
@@ -1171,10 +1275,24 @@ business, do check us out!
 
 Many thanks to Sam Tregar (author of the most excellent 
 HTML::Template module!) for his innumerable contributions 
-to this module over the past year, and most of all for getting 
+to this module over the years, and most of all for getting 
 me off my ass to finally get this thing up on CPAN!
 
 
+The following people have contributed specific suggestions or 
+patches which have helped improve CGI::Application --
+
+    Stephen Howard
+    Mark Stosberg
+
+
+Thanks also to all the members of the CGI-App mailing list!
+Your ideas, suggestions, insights (and criticism!) have helped
+shape this module immeasurably.  (To join the mailing list, simply
+send a blank message to "cgiapp-subscribe@lists.vm.com".)
+
+
+
 =head1 LICENSE
 
 Copyright (c) 2000, Jesse Erlbaum <jesse@vm.com> and 
